#/bin/bash

# --------------------------------------------------------------------------
# File:        backup_database.bash 
#
# Autor:       Martin Zahn, Akadia AG, 10.07.2008 
#
# Purpose:     Backup Oracle Database using RMAN (Physical Backup)
#              and DataPump (Logical  Backup)
#
# Note:        CREATE DIRECTORY <ORACLE_DATAPUMP_DIR> as User SYSTEM. 
#
# Usage:       rman_backup_database.bash 1>${BACKUP_LOG} 2>&1
#              as a Unix User which belongs to the DBA group
# --------------------------------------------------------------------------

# DEBUG
set -x

# Check that User belongs to the DBA Group
# ----------------------------------------
DBA_GROUP=`/usr/bin/id -G -n | grep "dba" | wc -l`
if [ "${DBA_GROUP}" != "1" ]
then
   exit 1
fi

# Keep Backup online for X days
# -----------------------------
BACKUP_WINDOW=1
export BACKUP_WINDOW 

# Setup ORACLE 10 Environment
# ---------------------------
export ORACLE_SID=GENTIC
export LISTENER_NAME=$ORACLE_SID
export ORACLE_BASE=/export/home/oracle
export ORACLE_HOME=$ORACLE_BASE/product/10.2.0
export ORACLE_DOC=$ORACLE_HOME/doc
export LD_LIBRARY_PATH=$ORACLE_HOME/lib:$LD_LIBRARY_PATH
export TNS_ADMIN=/export/home/oracle/admin/sqlnet
export NLS_LANG=american_america.al32utf8
export ORA_NLS10=$ORACLE_HOME/nls/data

# Set Backup Destination
# ----------------------
BACKUP_DEST=${ORACLE_BASE}/work/backup/${ORACLE_SID}
BACKUP_DATE=`date +%Y%m%d`; export BACKUP_DATE

# Set DataPump Parameters
# -----------------------
EXPORT_FILE=${BACKUP_DATE}_datapump_full_${ORACLE_SID}.dp
export EXPORT_FILE

EXPORT_LOG=${BACKUP_DATE}_datapump_full_${ORACLE_SID}.log
export EXPORT_LOG

ORACLE_DATAPUMP_DIR=dp${ORACLE_SID}; 
export ORACLE_DATAPUMP_DIR

# Backup Parameterfiles
# ---------------------
if [ -f ${ORACLE_HOME}/dbs/init${ORACLE_SID}.ora ]
then
   cp ${ORACLE_HOME}/dbs/init${ORACLE_SID}.ora ${BACKUP_DEST}/${BACKUP_DATE}_init${ORACLE_SID}.ora
fi
if [ -f ${ORACLE_HOME}/dbs/spfile${ORACLE_SID}.ora ]
then
   cp ${ORACLE_HOME}/dbs/spfile${ORACLE_SID}.ora ${BACKUP_DEST}/${BACKUP_DATE}_spfile${ORACLE_SID}.ora
fi

# Backup Passwordfile 
# -------------------
if [ -f ${ORACLE_HOME}/dbs/orapw${ORACLE_SID} ]
then
   cp ${ORACLE_HOME}/dbs/orapw${ORACLE_SID} ${BACKUP_DEST}/${BACKUP_DATE}_orapw${ORACLE_SID}
fi

# Start the RMAN physical backup 
# ------------------------------
${ORACLE_HOME}/bin/rman nocatalog target / <<-EOF 
   configure retention policy to redundancy ${BACKUP_WINDOW}; 
   configure retention policy to recovery window of ${BACKUP_WINDOW} days; 
   configure backup optimization on; 
   configure controlfile autobackup off; 
   configure default device type to disk; 
   configure device type disk parallelism 1 backup type to compressed backupset; 
   configure datafile backup copies for device type disk to 1; 
   configure channel device type Disk format '${BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_datafile_%s_%p.bak'; 
   configure maxsetsize to unlimited; 
   configure snapshot controlfile name to '${BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_snapshot_controlfile'; 
   set controlfile autobackup format for device type Disk to '${BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_controlfile_%F'; 
   show all; 

   run { 
     allocate channel ch1 type Disk maxpiecesize = 1900M; 
     backup full database noexclude 
     include current controlfile 
     format '${BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_datafile_%s_%p.bak' 
     tag 'datafile_daily'; 
   }

   run { 
     allocate channel ch1 type Disk maxpiecesize = 1900M; 
     backup archivelog all 
     delete all input 
     format '${BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_archivelog_%s_%p.bak' 
     tag 'archivelog_daily'; 
   } 

   run { 
      allocate channel ch1 type Disk maxpiecesize = 1900M; 
      backup format '${BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_controlfile_%s.bak' current controlfile; 
   } 

   crosscheck backup; 
   # restore controlfile validate; 
   # restore tablespace SYSTEM validate; 
   # restore archivelog all validate; 
   list backup of database; 
   report unrecoverable; 
   report schema; 
   report need backup; 
   report obsolete; 
   delete noprompt expired backup of database; 
   delete noprompt expired backup of controlfile;
   delete noprompt expired backup of archivelog all;
   delete noprompt obsolete recovery window of ${BACKUP_WINDOW} days;
   quit 
EOF

# Store the old Password for SYSTEM
# ---------------------------------
${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-SAVE_PASSWORD 1>/dev/null 2>&1
   set heading off
   set pagesize 0
   set tab off
   set trimspool on
   spool /tmp/old_password.lst
   SELECT password FROM dba_users WHERE UPPER(username) = 'SYSTEM';
   spool off;
SAVE_PASSWORD

OLD_PASSWOD=`cat /tmp/old_password.lst | sed -e 's/^$//' -e '/^ *$/d'`
export OLD_PASSWOD
rm -f /tmp/old_password.lst 1>/dev/null 2>&1

# Change temporarily the SYSTEM Password
# --------------------------------------
${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-TMP_PASSWORD 1>/dev/null 2>&1
   ALTER USER SYSTEM IDENTIFIED BY manager;
TMP_PASSWORD

# Start the DataPump logical backup
# ---------------------------------
cd ${BACKUP_DEST} 
ORACLE_DATAPUMP_USR_PWD="system/manager";
export ORACLE_DATAPUMP_USR_PWD
${ORACLE_HOME}/bin/expdp userid=${ORACLE_DATAPUMP_USR_PWD} directory=${ORACLE_DATAPUMP_DIR} parallel=5 full=y dumpfile=${EXPORT_FILE} logfile=${EXPORT_LOG}
gzip ${EXPORT_FILE}

# Reset the SYSTEM Password
# -------------------------
${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-RESET_PASSWORD 1>/dev/null 2>&1
   ALTER USER SYSTEM IDENTIFIED BY VALUES '${OLD_PASSWOD}';
RESET_PASSWORD

# Create TAR/GZIP file with all the backuped files 
# ------------------------------------------------
cd ${BACKUP_DEST} 
if [ "`pwd`" = "${BACKUP_DEST}" ]
then
   tar czvf ${BACKUP_DATE}_backupset_${ORACLE_SID}.tgz ${BACKUP_DATE}_*
   find . -type f -mtime +${BACKUP_WINDOW} -exec rm -rf {} \;
fi

exit; 
